TAG_PUNCTUATOR_MAP = {
    "O": (" ", False),
    "C": (", ", False),
    "P": (". ", True),
    "Q": ("? ", True),
    "E": ("! ", True),
}

DEFAULT_NER_MAPPING = {",": "C", ".": "P", "?": "Q", "!": "E"}
DIGIT_MASK = "<num>"


# byte format
NUM_BYTE_FORMAT = "!H"
LENGTH_BYTE_FORMAT = "!I"

NUM_BYTE_LENGTH = 2
LENGTH_BYTE_LENGTH = 4



import logging
import multiprocessing as mp
import signal
import struct
import threading
from threading import Thread
from time import sleep
from typing import List

from inference.inference_pipeline import InferenceServer
from utils.utils import register_logger

logger = logging.getLogger(__name__)
register_logger(logger)


class InferenceClient:
    """Inference client for communicating with punctuator server"""

    def __init__(self, conn, check_interval=0.1) -> None:
        self.conn = conn
        self.check_interval = check_interval

    def punctuation(self, inputs):
        self.conn.send(inputs)
        while True:
            try:
                if self.conn.poll(self.check_interval):
                    outputs = self.conn.recv()
                    return outputs
            except (struct.error, OSError) as err:
                logger.warning(f"{self.name} struct unpack error: {err}")
                raise err

    def terminate(self):
        """graceful shutdown everything"""
        logger.info("terminate the client")

        self.conn.close()


class Inference:
    """Interface for using the punctuator"""

    def __init__(
        self,
        inference_args,
        method="spawn",
        server_check_interval=0.1,
        task_check_interval=0.05,
        verbose=False,
    ) -> None:
        """Inference class for using the punctuator

        Args:
            inference_args (InferenceArguments): inference arguments
            method (str, optional): "fork" or "spawn". Defaults to "spawn".
            server_check_interval (float, optional): interval to check punctuator running status. Defaults to 0.1.
            task_check_interval (float, optional): interval to check new task. Defaults to 0.05.
            verbose (bool, optional): whether to ouput punctuation progress. Defaults to False.
        """
        self.termination = mp.get_context(method).Event()
        self.method = method
        self.inference_args = inference_args
        self.verbose = verbose

        self._init_termination()
        self._produce_server(task_check_interval)
        self.thread = Thread(target=self._run, args=(server_check_interval,))
        self.thread.start()

    def _produce_server(self, task_check_interval):
        logger.info("set up punctuator")
        self.c_conn, self.s_conn = mp.Pipe(True)
        server = InferenceServer(
            inference_args=self.inference_args,
            conn=self.s_conn,
            termination=self.termination,
            check_interval=task_check_interval,
            verbose=self.verbose,
        )
        self.server_process = mp.get_context(self.method).Process(
            target=server.run,
        )

        logger.info("start running punctuator")
        self.server_process.start()

        logger.info("start client")
        self.client = InferenceClient(conn=self.c_conn)

    def _init_termination(self):
        """init signal handler and termination event"""
        self.shutdown = threading.Event()
        signal.signal(signal.SIGTERM, self._terminate)
        signal.signal(signal.SIGINT, self._terminate)

    def _terminate(self, signum, frame):
        """graceful shutdown everything"""
        logger.info(f"[{signum}] terminate inference: {frame}")

        self.shutdown.set()
        self.termination.set()
        self.client.terminate()

    def _run(self, check_interval):
        while not self.shutdown.is_set():
            if self.server_process.exitcode is not None:
                logger.warning("punctuator is no longer working, restart")
                self._produce_server()
            sleep(check_interval)
        logger.info("terminate the punctuator")
        # self.server_process.terminate()

    def punctuation(self, inputs: List[str]):
        try:
            outputs = self.client.punctuation(inputs)
            return outputs
        except Exception as err:
            logger.error(f"error doing punctuation with details {str(err)}")
        return None

    def terminate(self):
        self.shutdown.set()
        self.termination.set()
        self.client.terminate()



import json
import logging
import struct
from functools import wraps
from itertools import filterfalse

import numpy as np
import torch
from pydantic import BaseModel
from transformers import DistilBertForTokenClassification, DistilBertTokenizerFast

from utils.constant import DIGIT_MASK, TAG_PUNCTUATOR_MAP
from utils.utils import register_logger

logger = logging.getLogger(__name__)
register_logger(logger)


def verbose(attr_to_log):
    def wrapper_out(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            self = func(self, *args, **kwargs)
            if self.verbose:
                logger.debug(
                    f'After {func.__name__}, {attr_to_log} is generated as "{getattr(self, attr_to_log)}"'
                )
            return self

        return wrapper

    return wrapper_out


class InferenceArguments(BaseModel):
    """Arguments pertaining to which model/config/tokenizer we are going to fine-tune from.

    Args:
        model_name_or_path(str): name or path of pre-trained model
        tokenizer_name(str): name of pretrained tokenizer
        tag2id_storage_path(str): tag2id storage path
    """

    model_name_or_path: str
    tokenizer_name: str
    tag2id_storage_path: str


# whole pipeline running in the seperate process, provide a function for user to call, use socket for communication
class InferencePipeline:
    """Pipeline for inference"""

    def __init__(self, inference_arguments, verbose=False):
        self.arguments = inference_arguments
        self.device = (
            torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        )
        self.tokenizer = DistilBertTokenizerFast.from_pretrained(
            self.arguments.tokenizer_name
        )
        self.classifer = DistilBertForTokenClassification.from_pretrained(
            inference_arguments.model_name_or_path
        )
        with open(inference_arguments.tag2id_storage_path, "r") as fp:
            tag2id = json.load(fp)
            self.id2tag = {id: tag for tag, id in tag2id.items()}

        self._reset_values()
        self.verbose = verbose

    @verbose("all_tokens")
    def pre_process(self, inputs):
        for input in inputs:
            input_tokens = input.split()
            digits = dict(
                list(filterfalse(lambda x: not x[1].isdigit(), enumerate(input_tokens)))
            )
            for index_key in digits.keys():
                input_tokens[index_key] = DIGIT_MASK
            self.digit_indexes.append(digits)
            self.all_tokens.append(input_tokens)
        return self

    @verbose("tokenized_input_ids")
    def tokenize(self):
        tokenized_inputs = self.tokenizer(
            self.all_tokens,
            is_split_into_words=True,
            padding=True,
            truncation=True,
            return_offsets_mapping=True,
            return_tensors="pt",
        )
        self.marks = self._mark_ignored_tokens(tokenized_inputs["offset_mapping"])
        self.tokenized_input_ids = tokenized_inputs["input_ids"].to(self.device)
        self.attention_mask = tokenized_inputs["attention_mask"].to(self.device)
        return self

    @verbose("argmax_preds")
    def classify(self):
        logits = self.classifer(self.tokenized_input_ids, self.attention_mask).logits
        if self.device.type == "cuda":
            self.argmax_preds = logits.argmax(dim=2).detach().cpu().numpy()
        else:
            self.argmax_preds = logits.argmax(dim=2).detach().numpy()
        return self

    @verbose("outputs")
    def post_process(self):
        reduce_ignored_marks = self.marks >= 0

        for pred, reduce_ignored, tokens, digit_index in zip(
            self.argmax_preds, reduce_ignored_marks, self.all_tokens, self.digit_indexes
        ):
            next_upper = True
            true_pred = pred[reduce_ignored]
            result_text = ""
            for id, (index, token) in zip(true_pred, enumerate(tokens)):
                tag = self.id2tag[id]
                if index in digit_index:
                    token = digit_index[index]
                if next_upper:
                    token = token.capitalize()
                punctuator, next_upper = TAG_PUNCTUATOR_MAP[tag]
                result_text += token + punctuator
            self.outputs.append(result_text.strip())

        return self

    def punctuation(self, inputs):
        self._reset_values()
        return self.pre_process(inputs).tokenize().classify().post_process().outputs

    def _mark_ignored_tokens(self, offset_mapping):
        samples = []
        for sample_offset in offset_mapping:
            # create an empty array of -100
            sample_marks = np.ones(len(sample_offset), dtype=int) * -100
            arr_offset = np.array(sample_offset)

            # set labels whose first offset position is 0 and the second is not 0, only special tokens second is also 0
            sample_marks[(arr_offset[:, 0] == 0) & (arr_offset[:, 1] != 0)] = 0
            samples.append(sample_marks.tolist())

        return np.array(samples)

    def _reset_values(self):
        self.digit_indexes = []
        self.all_tokens = []
        self.outputs = []


class InferenceServer:
    """Inference server"""

    def __init__(
        self, inference_args, conn, termination, check_interval, verbose=False
    ) -> None:
        """Server for receiving tasks from client and do punctuation

        Args:
            server_address (str, optional): [server socket address]. Defaults to "/tmp/socket".
        """
        self.inference_pipeline = InferencePipeline(inference_args, verbose=verbose)
        self.conn = conn
        self.termination = termination
        self.check_interval = check_interval

    # data structure: |num|length|text|length|text...
    def punctuation(self):
        try:
            inputs = self.conn.recv()
            outputs = self.inference_pipeline.punctuation(inputs)
            self.conn.send(outputs)
        except OSError as err:
            logger.warning(f"error receiving inputs: {err}")
        except struct.error as err:
            logger.warning(f"struct unpack error: {err}")

    def run(self):
        assert self.inference_pipeline, "no inference pipeline set up"
        logger.info("server is running")
        while True:
            try:
                if (
                    self.conn.poll(self.check_interval)
                    and not self.termination.is_set()
                ):
                    self.punctuation()
                if self.termination.is_set():
                    logger.info("termination is set")
                    break
            except (struct.error, OSError) as err:
                logger.warning(f"struct unpack error: {err}")
                raise err
            except KeyboardInterrupt:
                logger.warning("punctuator shut down by keyboard interrupt")
                break
        self.terminate()

    def terminate(self):
        logger.info("terminate punctuation server")

        self.conn.close()



import pandas as pd
from tqdm import tqdm

from data_process.data_cleanning import dataframe_data_cleaning
from utils.constant import DEFAULT_NER_MAPPING, DIGIT_MASK


def cleanup_data_from_csv(
    csv_path,
    target_col,
    output_file_path,
    ner_mapping=DEFAULT_NER_MAPPING,
    additional_to_remove=[],
    special_cleaning_funcs=[],
):
    """clean up training data from csv file

    Args:
        csv_path (string): path of training data csv
        target_col (string): column of training data in csv
        output_file_path (string): path of cleaned data
        ner_mapping (dict, optional): NER mapping of punctuation marks. Defaults to utils.constant.DEFAULT_NER_MAPPING
        additional_to_remove (list, optional): additional special characters to remove, default []
        special_cleaning_funcs (List[func], optional): additional cleaning funcs to apply to csv data, default []
    """
    dataframe = pd.read_csv(csv_path)
    additional_to_remove = ["—"]
    kept_punctuations = set(ner_mapping.keys())
    result_df = dataframe_data_cleaning(
        dataframe[1500:],
        target_col,
        kept_punctuations,
        additional_to_remove,
        *special_cleaning_funcs,
    )
    with open(output_file_path, "w+") as output_file:
        for row in result_df[target_col].tolist():
            if row:
                if row[-1] not in ner_mapping:
                    output_file.write("%s. \n" % row)
                else:
                    output_file.write("%s \n" % row)


def process_line(line, ner_mapping=DEFAULT_NER_MAPPING):
    text_list = line.split()
    word_list = []
    token_list = []
    # clean up puncs in the beginning of the text
    latest_word = text_list.pop(0)
    while latest_word in ner_mapping:
        if not text_list:
            break
        latest_word = text_list.pop(0)
    latest_token = "O"
    latest_is_punc = False
    for word in text_list:
        if word in ner_mapping:
            if not latest_is_punc:
                latest_token = ner_mapping[word]
                latest_is_punc = True
                word_list.append(latest_word)
                token_list.append(latest_token)
            else:
                pass
        else:
            if not latest_is_punc:
                word_list.append(latest_word)
                token_list.append(latest_token)
            latest_is_punc = False
            if word.isdigit():
                word = DIGIT_MASK
            latest_word = word
            latest_token = "O"
    if not latest_is_punc:
        word_list.append(latest_word)
        token_list.append(latest_token)
    return word_list, token_list


def generate_training_data(cleaned_data_path, training_data_path):
    """generate "token tag" format training data based on cleaned text

    Args:
        cleaned_data_path (string): path of cleaned data
        training_data_path (string): path of generated training data
    """
    with open(cleaned_data_path, "r") as data_file:
        lines = data_file.readlines()
    with open(training_data_path, "w+") as training_data_file:
        pbar = tqdm(lines)
        for line in pbar:
            words, tokens = process_line(line)
            for word, token in zip(words, tokens):
                training_data_file.write("%s\t%s\n" % (word, token))
        pbar.close()



import logging
import sys


class ColorfulFormatter(logging.Formatter):
    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    red = "\x1b[31;21m"
    green = "\x1b[32m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    formatter = (
        "%(asctime)s - {color}%(levelname)s{reset} - %(filename)s:%(lineno)d"
        " - %(module)s.%(funcName)s - %(process)d - %(message)s"
    )
    FORMATS = {
        logging.DEBUG: formatter.format(color=grey, reset=reset),
        logging.INFO: formatter.format(color=green, reset=reset),
        logging.WARNING: formatter.format(color=yellow, reset=reset),
        logging.ERROR: formatter.format(color=red, reset=reset),
        logging.CRITICAL: formatter.format(color=bold_red, reset=reset),
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def register_logger(logger=None):
    """register colorful debug log"""
    if not logger:
        logger = logging.getLogger()
    if not logger.hasHandlers():
        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(ColorfulFormatter())
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)


def recv_all(conn, length):
    buffer = bytearray(length)
    mv = memoryview(buffer)
    size = 0
    while size < length:
        packet = conn.recv_bytes_into(mv)
        mv = mv[packet:]
        size += packet
    return buffer



import argparse
import os

from transformers import DistilBertForTokenClassification


def upload_model(fine_tuned_model_path):
    fine_tuned_model = DistilBertForTokenClassification.from_pretrained(
        fine_tuned_model_path
    )
    token = os.environ["HUGGINGFACE_TOKEN"]

    fine_tuned_model.push_to_hub("distilbert_punctuator_en", use_auth_token=token)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--fine_tuned_model_path", default="")
    args = parser.parse_args()

    upload_model(fine_tuned_model_path=args.fine_tuned_model_path)



from string import punctuation

from tqdm import tqdm

tqdm.pandas()

default_kept_punctuations = {",", ".", "?", "!"}


def dataframe_data_cleaning(
    df, target_col, kept_punctuations, additional_to_remove, *special_cleaning_funcs
):
    """
    Clean up data in dataframe by removing all special characters except kept ones.
    """
    if special_cleaning_funcs:
        for func in special_cleaning_funcs:
            df[target_col] = df[target_col].progress_apply(lambda x: func(x))

    removed_punctuations = "".join(
        [p for p in punctuation if p not in kept_punctuations] + additional_to_remove
    )
    translator = str.maketrans("", "", removed_punctuations)
    space_translator = str.maketrans(
        {key: " {0} ".format(key) for key in kept_punctuations}
    )

    df[target_col] = df[target_col].progress_apply(
        lambda x: x.lower().translate(translator).translate(space_translator).strip()
    )

    df.dropna(subset=[target_col])
    return df



import json
import logging
import time
from typing import Dict, Optional

import numpy as np
import torch
from pydantic import BaseModel
from torch._C import device  # noqa: F401
from torch.utils.data import DataLoader, Dataset
from tqdm import tqdm
from transformers import (
    AdamW,
    DistilBertConfig,
    DistilBertForTokenClassification,
    DistilBertTokenizerFast,
)

from training.dataset import generate_tag_ids, read_data, train_test_split

logger = logging.getLogger(__name__)


class TrainingArguments(BaseModel):
    """Arguments pertaining to which model/config/tokenizer we are going to fine-tune from.

    Args:
        data_file_path(str): path of training data
        model_name(str): name or path of pre-trained model
        tokenizer_name(str): name of pretrained tokenizer
        split_rate(float): train and validation split rate
        sequence_length(int): sequence length of one sample
        epoch(int): number of epoch
        batch_size(int): batch size
        model_storage_path(str): fine-tuned model storage path
        tag2id_storage_path(str): tag2id storage path
        addtional_model_config(Optional[Dict]): additional configuration for model
    """

    data_file_path: str
    model_name: str
    tokenizer_name: str
    split_rate: float
    sequence_length: int
    epoch: int
    batch_size: int
    model_storage_path: str
    tag2id_storage_path: str
    addtional_model_config: Optional[Dict]


class TrainingPipeline:
    def __init__(self, training_arguments):
        """Training pipeline for fine-tuning the distilbert token classifier for punctuation

        Args:
            training_arguments (TrainingArguments): arguments passed to training pipeline
        """
        self.arguments = training_arguments
        self.device = (
            torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        )

    def load_training_data(self):
        logger.info("load training data")
        texts, tags = read_data(
            self.arguments.data_file_path, self.arguments.sequence_length
        )
        (
            self.train_texts,
            self.val_texts,
            self.train_tags,
            self.val_tags,
        ) = train_test_split(texts, tags, test_size=self.arguments.split_rate)
        self.tag2id, self.id2tag = generate_tag_ids(tag_docs=tags)

        return self

    def tokenize(self):
        logger.info("tokenize data")
        tokenizer = DistilBertTokenizerFast.from_pretrained(
            self.arguments.tokenizer_name
        )
        self.train_encodings = tokenizer(
            self.train_texts,
            is_split_into_words=True,
            return_offsets_mapping=True,
            padding=True,
            truncation=True,
        )
        self.val_encodings = tokenizer(
            self.val_texts,
            is_split_into_words=True,
            return_offsets_mapping=True,
            padding=True,
            truncation=True,
        )
        self.train_labels = self._encode_tags(self.train_tags, self.train_encodings)
        self.val_labels = self._encode_tags(self.val_tags, self.val_encodings)

        return self

    def generate_dataset(self):
        logger.info("generate dataset from tokenized data")
        self.train_encodings.pop("offset_mapping")
        self.val_encodings.pop("offset_mapping")
        self.training_dataset = PunctuatorDataset(
            self.train_encodings, self.train_labels
        )
        self.val_dataset = PunctuatorDataset(self.val_encodings, self.val_labels)

        return self

    def fine_tune(self):
        logger.info("start fine tune")
        config = DistilBertConfig.from_pretrained(
            self.arguments.model_name,
            label2id=self.tag2id,
            id2label=self.id2tag,
            num_labels=len(self.tag2id),
            **self.arguments.addtional_model_config,
        )
        self.classifier = DistilBertForTokenClassification.from_pretrained(
            self.arguments.model_name, config=config
        )
        self.classifier.to(self.device)

        train_loader = DataLoader(
            self.training_dataset, batch_size=self.arguments.batch_size, shuffle=True
        )
        val_loader = DataLoader(
            self.val_dataset, batch_size=self.arguments.batch_size, shuffle=True
        )
        optim = AdamW(self.classifier.parameters(), lr=5e-5)

        best_valid_loss = 1
        with tqdm(total=self.arguments.epoch) as pbar:
            for epoch in range(self.arguments.epoch):
                pbar.set_description(f"Processing epoch: {epoch + 1}")

                start_time = time.time()

                self.classifier.train()

                train_loss, train_acc = self._train(train_loader, optim)
                val_loss, val_acc = self._train(val_loader, optim, True)

                end_time = time.time()

                epoch_mins, epoch_secs = self._epoch_time(start_time, end_time)

                if val_loss < best_valid_loss:
                    best_valid_loss = val_loss
                    self.best_state_dict = self.classifier.state_dict()

                logger.info(
                    f"Epoch: {epoch + 1:02} | Epoch Time: {epoch_mins}m {epoch_secs}s"
                )
                logger.info(
                    f"\tTrain Loss: {train_loss:.3f} | Train Acc: {train_acc * 100:.2f}%"
                )
                logger.info(
                    f"\t Val. Loss: {val_loss:.3f} |  Val. Acc: {val_acc * 100:.2f}%"
                )
                pbar.update(1)

            logger.info("fine-tune finished")

        return self

    def persist(self):
        """Persist this model into the passed directory."""

        logger.info("persist fine-tuned model")

        self.classifier.load_state_dict(self.best_state_dict)
        self.classifier.save_pretrained(self.arguments.model_storage_path)

        # persist model parameters
        json.dump(self.tag2id, open(self.arguments.tag2id_storage_path, "w"), indent=4)

        logger.info(f"fine-tuned model stored to {self.arguments.model_storage_path}")

    def _encode_tags(self, tags, encodings):
        logger.info("encoding tags")
        labels = [[self.tag2id[tag] for tag in doc] for doc in tags]
        encoded_labels = []
        with tqdm(total=len(labels)) as pbar:
            for doc_labels, doc_offset in zip(labels, encodings.offset_mapping):
                # create an empty array of -100
                doc_enc_labels = np.ones(len(doc_offset), dtype=int) * -100
                arr_offset = np.array(doc_offset)

                # set labels whose first offset position is 0 and the second is not 0
                doc_enc_labels[
                    (arr_offset[:, 0] == 0) & (arr_offset[:, 1] != 0)
                ] = doc_labels
                encoded_labels.append(doc_enc_labels.tolist())
                pbar.update(1)

        return encoded_labels

    def _train(self, iterator, optim, is_val=False):
        epoch_loss = 0
        epoch_acc = 0
        if is_val:
            self.classifier.train(False)
        else:
            self.classifier.train()

        steps = 0
        with tqdm(total=len(iterator)) as pbar:
            for batch in iterator:
                steps += 1
                pbar.set_description(f"Processing batch: {steps}")

                optim.zero_grad()
                input_ids = batch["input_ids"].to(self.device)
                attention_mask = batch["attention_mask"].to(self.device)
                labels = batch["labels"].to(self.device)
                outputs = self.classifier(
                    input_ids, attention_mask=attention_mask, labels=labels
                )
                loss = outputs.loss
                logits = outputs.logits

                epoch_loss += loss.item()
                epoch_acc += self._accuracy(logits, attention_mask, labels)

                if not is_val:
                    loss.backward()
                    optim.step()

                pbar.update(1)

        return epoch_loss / steps, epoch_acc / steps

    def _epoch_time(self, start_time, end_time):
        elapsed_time = end_time - start_time
        elapsed_mins = int(elapsed_time // 60)
        elapsed_secs = int(elapsed_time - (elapsed_mins * 60))
        return elapsed_mins, elapsed_secs

    def _accuracy(self, logits, attention_mask, labels):
        if self.device.type == "cuda":
            max_preds = logits.argmax(dim=2).detach().cpu().numpy().flatten()
            flattened_labels = labels.detach().cpu().numpy().flatten()
            flattened_attention = attention_mask.detach().cpu().numpy().flatten()
        else:
            max_preds = logits.argmax(dim=2).detach().numpy().flatten()
            flattened_labels = labels.detach().numpy().flatten()
            flattened_attention = attention_mask.detach().numpy().flatten()

        not_padding_labels = flattened_labels[flattened_attention == 1]
        not_padding_preds = max_preds[flattened_attention == 1]
        reduce_ignored = not_padding_labels >= 0
        true_labels = not_padding_labels[reduce_ignored]  # remove ignored -100
        true_preds = not_padding_preds[reduce_ignored]

        if true_preds.shape[0] == true_labels.shape[0]:
            return np.sum(true_preds == true_labels) / true_preds.shape[0]

        return 0

    def run(self):
        self.load_training_data().tokenize().generate_dataset().fine_tune().persist()


class PunctuatorDataset(Dataset):
    def __init__(self, encodings, labels):
        self.encodings = encodings
        self.labels = labels

    def __getitem__(self, idx):
        item = {key: torch.tensor(val[idx]) for key, val in self.encodings.items()}
        item["labels"] = torch.tensor(self.labels[idx])
        return item

    def __len__(self):
        return len(self.labels)


def encode_tags(labels, encodings):
    encoded_labels = []
    for doc_labels, doc_offset in zip(labels, encodings.offset_mapping):
        # create an empty array of -100
        doc_enc_labels = np.ones(len(doc_offset), dtype=int) * -100
        arr_offset = np.array(doc_offset)

        # set labels whose first offset position is 0 and the second is not 0, only special tokens second is also 0
        doc_enc_labels[(arr_offset[:, 0] == 0) & (arr_offset[:, 1] != 0)] = doc_labels
        encoded_labels.append(doc_enc_labels.tolist())

    return encoded_labels



import random
from typing import List

from tqdm import tqdm

PAD_TOKEN = "[PAD]"
NORMAL_TOKEN_TAG = "O"


def read_data(file_path, sequence_length) -> List[List]:
    def read_line(text_line):
        return text_line.strip().split("\t")

    token_docs = []
    tag_docs = []
    line_index = 0

    token_doc = []
    tag_doc = []
    with open(file_path, "r") as data_file:
        pbar = tqdm(data_file.readlines())
        for line in pbar:
            if line_index == 0:
                token_doc = []
                tag_doc = []
            processed_line = read_line(line)
            token_doc.append(processed_line[0])
            tag_doc.append(processed_line[1])
            line_index += 1
            if line_index == sequence_length:
                token_docs.append(token_doc)
                tag_docs.append(tag_doc)
                line_index = 0
                pbar.update(sequence_length)
        token_doc += [PAD_TOKEN] * (sequence_length - line_index)
        tag_doc += [NORMAL_TOKEN_TAG] * (sequence_length - line_index)
        token_docs.append(token_doc)
        tag_docs.append(tag_doc)

        pbar.close()

    return token_docs, tag_docs


def generate_tag_ids(tag_docs):
    unique_tags = set([tag for tags in tag_docs for tag in tags])
    tag2id = {tag: id for id, tag in enumerate(unique_tags)}
    id2tag = {id: tag for tag, id in tag2id.items()}

    return tag2id, id2tag


def train_test_split(tokens, tags, test_size=0.2, shuffle=True):
    if shuffle:
        random.shuffle(tokens)
        random.shuffle(tags)
    index = round(len(tokens) * (1 - test_size))
    return tokens[:index], tags[:index], tokens[index:], tags[index:]



import re


def remove_brackets_text(input):
    """remove brackets and text inside

    Args:
        input (string): text to apply the regex func to

    Returns:
        [type]: [description]
    """
    return re.sub(r"\([^()]*\)", " ", input)


def keep_only_latin_characters(input):
    """keep only latin characters

    Args:
        input (string): text to apply the regex func to

    Returns:
        [type]: [description]
    """
    regex = re.compile("[^\u0020-\u024F]")
    return regex.sub("", input)



from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

with open(path.join(here, "requirements.txt"), encoding="utf-8") as f:
    requires = [req.strip() for req in f if req]

setup(
    name="distilbert-punctuator",
    version="0.1.0",
    description="A small seq2seq punctuator tool based on DistilBERT",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Zhong Qishuai",
    author_email="ferdinandzhong@gmail.com",
    url="https://https://github.com/FerdinandZhong/punctuator",
    packages=find_packages(exclude=["tests*", "example*"]),
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">3.6",
    install_requires=requires,
    extras_require={
        "data_process": ["pandas>=1.3.4"],
        "dev": [
            "pytest>=6",
            "flake8>=3.8",
            "black>=20.8b1",
            "isort>=5.6",
            "autoflake>=1.4",
        ],
    },
    zip_safe=False,
    entry_points={
        "console_scripts": [],
    },
)


from .additional_data_process import keep_only_latin_characters, remove_brackets_text
from .data_process import cleanup_data_from_csv, generate_training_data

__all__ = [
    "cleanup_data_from_csv",
    "generate_training_data",
    "remove_brackets_text",
    "keep_only_latin_characters",
]
